/* ar clone for QNX4 w/the watcom compiler */
/* this probably should be done in perl */
/* Michael Hunter, bagpiper@netcom.com, QUICS: mphunter */
/* I have vaguely followed 1003.2 (POSIX .2), part 6.1 (ar) */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

const char *options= "cdprqtuvx" ;
char *archive= NULL ;
char buffer[1024] ;

/* options */
int c= 0 ;
int d= 0 ;
int p= 0 ;
int r= 0 ;
int t= 0 ;
int u= 0 ;
int v= 0 ;
int x= 0 ;

char node[_MAX_NODE] ;
char dir[_MAX_DIR] ;
char fname[_MAX_FNAME] ;
char ext[_MAX_EXT] ;

static void usage(void) ;

int main(int argc, char *argv[])
{
	char **files= NULL ;
	int num_files= 0 ;
	int current_file= 0 ;
	int ch, i ;

	while((ch= getopt(argc, argv, options)) != -1)
	{
		switch(ch)
		{
			case 'c': 
				c= 1 ;
				break ;

			case 'd':
				d= 1 ;
				break ;

			case 'p':
				p= 1 ;
				break ;

			case 'r':
				r= 1 ;
				break ;

			case 't':
				t= 1 ;
				break ;

			case 'u':
				u= 1 ;
				break ;

			case 'v':
				v= 1 ;
				break ;

			case 'x':
				x= 1 ;
				break ;

			default:
				fprintf(stderr, "don't understand option %c\n", ch) ;
				usage() ;
				exit(-1) ;
		}
	}	

	if ((argc > 1) && (optind == 1)&&(strcmp(argv[1], "q")==0))
	{
		r= 1 ;
		optind++ ;
	}

	if (argv[optind] == NULL)
	{
		fprintf(stderr, "Need the name of an archive.\n") ;
		usage() ;
		exit(-1) ;
	} else
	{
		 archive= argv[optind] ;
	}
	optind++ ;

	while(argv[optind] != NULL)
	{
		if (files == NULL)
		{
			files= malloc(sizeof(char *)*(num_files= 10)) ;
			if (files == NULL)
			{
				fprintf(stderr, "Not enough memory.\n") ;
				exit(-1) ;
			}
		}

		if (current_file == num_files)
		{
			char **newfiles= realloc(files, (num_files *= 1.5)*sizeof(char *)) ;
			if (newfiles == NULL)
			{
				fprintf(stderr, "Not enougth memory.\n") ;
				free(files) ;
				exit(-1) ;
			} else
			{
				files= newfiles ;
			}
		}

		files[current_file++]= argv[optind++] ;
	}

	if ((d && p) || (d && r) || (d && t) || (d && x))
	{
		fprintf(stderr, "d can't be used with p, r, t, or x.\n") ;
		usage() ;
		exit(-1) ;
	}

	if ((p && r) || (p && t) || (p && x))
	{
		fprintf(stderr, "p can't be used with d, r, t, or x.\n") ;
		usage() ;
		exit(-1) ;
	}

	if ((r && t) || (r && x))
	{
		fprintf(stderr, "r can't be used with d, p, t, or x.\n") ;
		usage() ;
		exit(-1) ;
	}

	if (t && x)
	{
		fprintf(stderr, "t can't be used with d, p, r, or x.\n") ;
		usage() ;
		exit(-1) ;
	}

	/* now we now unique... */
	/* remember that current_file is pointing one past the last real file */
	/* num_files is the number of allocated (vs. used) slots in the files array */
	if (d)
	{
		if (current_file == 0)
		{
			fprintf(stderr, "delete what?\n") ;
			usage() ;
			exit(-1) ;
		}

		for(i= 0;i < current_file; i++)
		{
			/* what hell does extension mean in POSIX land.... */
			_splitpath(files[i], node, dir, fname, ext) ;
			sprintf(buffer, "wlib %s -%s%s",archive, fname, ext) ; 

			/* spawn would be more efficent since not loading shell... */
			if (system(buffer) == -1)
			{
				fprintf(stderr, "error loading shell to run wlib.\n") ;
				exit(-1) ;
			}
		}
		return 0 ; 
 	}

	if (p || t) /* I ignore the file args, etc. in 6.1.2 */
	{
		sprintf(buffer, "wlib %s", archive) ;
		if (system(buffer) == -1)
		{
			fprintf(stderr, "problem loading shell for wlib.\n") ;
			exit(-1) ;
		}
		return 0 ;
	}

	if (r) /* I ignore the order restrictions in 6.1.3.. */ 
	{
		if (current_file == 0)
		{
			fprintf(stderr, "replace what?\n") ;
			usage() ;
			exit(-1) ;
		}

		for(i= 0;i < current_file; i++)
		{
			/* what hell does extension mean in POSIX land.... */
			_splitpath(files[i], node, dir, fname, ext) ;
			sprintf(buffer, "wlib %s -+%s%s",archive, fname, ext) ; 

			/* spawn would be more efficent since not loading shell... */
			if (system(buffer) == -1)
			{
				fprintf(stderr, "error loading shell to run wlib.\n") ;
				exit(-1) ;
			}
		}
		return 0 ;
 	}
	
	if (x) /* ignore file name length stuff, etc */
	{
		if (current_file == 0)
		{
			sprintf(buffer, "wlib -x %s\n", archive) ;
			if (system(buffer) == -1)
			{
				fprintf(stderr, "problem loading shell for wlib.\n") ;
				exit(-1) ;
			}
		} else
		{
			for(i= 0;i < current_file; i++)
			{
				/* what hell does extension mean in POSIX land.... */
				_splitpath(files[i], node, dir, fname, ext) ;
				sprintf(buffer, "wlib %s :%s%s",archive, fname, ext) ; 

				/* spawn would be more efficent since not loading shell... */
				if (system(buffer) == -1)
				{
					fprintf(stderr, "error loading shell to run wlib.\n") ;
					exit(-1) ;
				}
			}
		}
		return 0 ;
 	}
	return 0 ; /* keep the compiler from complaining */
}
	
static void usage(void)
{
	printf("usage for ar (library manager):\n") ;
	printf("\tar -d [-v] archive file ...\n") ;
	printf("\tar -p [-v] archive [file ...]\n") ;
	printf("\tar -r [-cuv] archive file ...\n") ;
	printf("\tar -q [-cub] archive file ...\n") ;
	printf("\tar -t [-v] archive [file ...]\n") ;
	printf("\tar -x [-v] archive [file ...]\n") ;

	printf("\t-v, -c, and -u are ignored.\n") ;
	printf("\tar -q is not POSIX, but seems to be common.\n") ;
	printf("\tar q does work, but q must be the second argument!!\n") ;
}		